#include <proto.h>

#include "gtprogbar.h"
#include "gtapp.h"
#include "gtcommon.h"
#include "gtresname.h"
#include "gterror.h"

/************************************************************************\
*
* GtProgBar class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init widget.
*
\************************************************************************/
GtProgBar *GtMkProgBar(char *n,GtManager *p,GtTag *tgs)
{
	GtProgBar *obj;
	
	if(!(obj=new GtProgBar()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtProgBar object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init widget.
*
\************************************************************************/
GtProgBar *GtMkProgBar(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtProgBar *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkProgBar(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty progbar.
*
\************************************************************************/
GtProgBar::GtProgBar(void)
{
	start=0;
	range=100;
	val=0;
	orient=GTO_HORIZ;
	bdrw=1;
	minxs=6;
	minys=6;
}

/************************************************************************\
*
* Setup progbar.
*
\************************************************************************/
sbool GtProgBar::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	int s=0,e=100;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_ORIENT,GTRT_ORIENT,&orient);
	app->GetRes(this,GTR_BDR,GTRT_BYTE,&bdrw);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Orient:
				orient=t->d.val;
				if(orient>GTO_LAST_NR)
					orient=GTO_HORIZ;
				break;
			case GT_Start:
				s=(int)t->d.val;
				break;
			case GT_End:
				e=(int)t->d.val;
				break;
			case GT_Value:
				val=t->d.val;
				break;
			case GT_Border:
				bdrw=t->d.val;
				break;
		}
	}
	start=GTMIN(s,e);
	e=e-s;
	if(e<0)
		range=-e;
	else
		range=e;
	if((val<start) || (val>start+(int)range))
		val=start;
	this->SetMinMaxSize(2*bdrw+4,2*bdrw+4,GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set progbar attributes.
*
\************************************************************************/
sbool GtProgBar::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE,rch=FALSE,bch=FALSE;
	int s,e;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	s=start;
	e=start+range;
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Orient:
				orient=t->d.val;
				if(orient>GTO_LAST_NR)
					orient=GTO_HORIZ;
				ch=TRUE;
				break;
			case GT_Start:
				s=(int)t->d.val;
				ch=TRUE;
				rch=TRUE;
				break;
			case GT_End:
				e=(int)t->d.val;
				ch=TRUE;
				rch=TRUE;
				break;
			case GT_Value:
				val=t->d.val;
				ch=TRUE;
				break;
			case GT_Border:
				bdrw=t->d.val;
				bch=TRUE;
				break;
		}
	}
	if(ch)
	{
		if(rch)
		{
			start=GTMIN(s,e);
			e=e-s;
			if(e<0)
				range=-e;
			else
				range=e;
		}
		if((val<start) || (val>start+(int)range))
			val=start;
	}
	if(bch)
		this->SetMinMaxSize(2*bdrw+4,2*bdrw+4,GTS_MAX,GTS_MAX);
	if(ch || bch)
		this->Refresh();
	return(TRUE);
}

/************************************************************************\
*
* Get progbar attributes.
*
\************************************************************************/
void GtProgBar::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Orient:
				*ptr=orient;
				break;
			case GT_Start:
				*ptr=start;
				break;
			case GT_End:
				*ptr=start+range;
				break;
			case GT_Value:
				*ptr=val;
				break;
			case GT_Border:
				*ptr=bdrw;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtProgBar::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtProgBar")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Refesh progbar.
*
\************************************************************************/
void GtProgBar::Refresh(void)
{
	int xp=0,yp=0;
	
	if(!gfx)
		return;
	if(flags & GTWF_NOWIN)
	{
		xp=xpos;
		yp=ypos;
	}
	if(range)
	{
		if(orient==GTO_HORIZ)
		{
			gtwin->Clear(xp+bdrw+(xsize-2*bdrw)*(val-start)/range,yp+bdrw,
								xsize-2*bdrw-(xsize-2*bdrw)*(val-start)/range,ysize-2*bdrw);
		}
		else
		{
			gtwin->Clear(xp+bdrw,yp+bdrw,xsize-2*bdrw,
									ysize-2*bdrw-(ysize-2*bdrw)*(val-start)/range);
		}
	}
	this->Draw(NULL);
}

/************************************************************************\
*
* Set progbar event mask.
*
\************************************************************************/
void GtProgBar::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask);
}

/************************************************************************\
*
* Draw progbar.
*
\************************************************************************/
sbool GtProgBar::Draw(XEvent *xe)
{
	int n,xp,yp;
	ulong sz;
	
	if(xe && ((xe->type!=Expose) || (xe->xexpose.count!=0)))
		return(TRUE);
	if(flags & GTWF_NOWIN)
	{
		xp=xpos;
		yp=ypos;
	}
	else
	{
		xp=0;
		yp=0;
	}
	if(orient==GTO_HORIZ)
	{
		if(range)
			sz=(xsize-2*bdrw)*(val-start)/range;
		else
			sz=(xsize-2*bdrw);
		gtfill->DrawRectangle(gtwin,xp+bdrw,yp+bdrw,sz,ysize-2*bdrw,TRUE);
	}
	else
	{
		if(range)
			sz=(ysize-2*bdrw)*(val-start)/range;
		else
			sz=(ysize-2*bdrw);
		gtfill->DrawRectangle(gtwin,xp+bdrw,yp+ysize-2*bdrw-sz,xsize-2*bdrw,sz,TRUE);
	}
	for(n=0; n<bdrw; n++)
	{
		GT3DBOX(gtdrk,gtlht,gtwin,xp+n,yp+n,xsize-2*n,ysize-2*n);
	}
	return(TRUE);
}
